@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import argparse
+import logging
 import sys
 import os.path
 from glob import glob
@@ -53,7 +54,7 @@ def get_env_yaml(build_config_yaml, product_yaml):
     if build_config_yaml is None or product_yaml is None:
         return None
 
-    env_yaml = ssg.yaml.open_environment(build_config_yaml, product_yaml)
+    env_yaml = ssg.environment.open_environment(build_config_yaml, product_yaml)
     return env_yaml
 
 
@@ -62,8 +63,8 @@ def get_profile_files_from_root(env_yaml, product_yaml):
     if env_yaml:
         base_dir = os.path.dirname(product_yaml)
         profiles_root = ssg.utils.required_key(env_yaml, "profiles_root")
-        profile_files = glob("{base_dir}/{profiles_root}/*.profile"
-                             .format(profiles_root=profiles_root, base_dir=base_dir))
+        profile_files = sorted(glob("{base_dir}/{profiles_root}/*.profile"
+                             .format(profiles_root=profiles_root, base_dir=base_dir)))
     return profile_files
 
 
@@ -72,6 +73,13 @@ def main():
     args = parser.parse_args()
     env_yaml = get_env_yaml(args.build_config_yaml, args.product_yaml)
 
+    build_root = os.path.dirname(args.build_config_yaml)
+
+    logfile = "{build_root}/{product}/control_profiles.log".format(
+            build_root=build_root,
+            product=env_yaml["product"])
+    logging.basicConfig(filename=logfile, level=logging.INFO)
+
     if args.controls_dir:
         controls_manager = ssg.controls.ControlsManager(args.controls_dir, env_yaml)
         controls_manager.load()
